const AWS = require('aws-sdk');
const Logger = require('./logger');

AWS.config.region = process.env.ES_REGION;

const elasticsearch = require('elasticsearch');
const httpAwsEs = require('http-aws-es');

const es = elasticsearch.Client({
  hosts: [process.env.ES_ENDPOINT],
  connectionClass: httpAwsEs,
  amazonES: {
    region: process.env.ES_REGION,
  },
});

const isIndexExists = index => es.indices.exists({ index });

const createIndex = index => es.indices.create({ index })
  .then(response => {
    Logger.debug(`Index "${index}" was successfully created`);
    return response;
  })

const makeSureIndexIsOnPlace = (index, createIndexIfNeeded) => {
  Logger.debug(`Making sure index "${index}" exists`);
  if (!createIndexIfNeeded) {
    return Promise.resolve();
  }
  return isIndexExists(index)
    .then(exists => {
      if (exists === false) {
        throw new Error('Index does not exist.')
      }
    })
    .catch(() => {
      Logger.debug(`Index "${index}" does not exist, creating it`);
      return createIndex(index)
    });
};

const createHandlers = (index, type) => ({
  INSERT: (id, doc) => es.create({
    index,
    id,
    body: doc,
    type,
  }),
  MODIFY: (id, doc) => es.update({
        index,
        id,
        body: {
          doc,
        },
        type,
      }).catch(error => {
        if (/.*document_missing_exception.*/.test(error.message)) {
          console.log(`Document is missing, will create it. (${id})`);
          return es.create({
            index,
            id,
            body: doc,
            type,
          })
        }
        throw new Error(error)
      }),
  REMOVE: id => es.delete({
    index,
    id,
    type,
  }),
});

module.exports = ({
  index,
  type,
  createIndexIfNeeded,
}) => makeSureIndexIsOnPlace(index, createIndexIfNeeded)
  .then(() => createHandlers(index, type));
